# automatically generated by the FlatBuffers compiler, do not modify

# namespace: AOC

class ObjectState(object):
    Ready = 0
    Founded = 1
    Alive = 2
    Dying = 3
    Dead = 4
    Rotting = 5
    Unknown = 6
    Dopple = 7
    Reusable = 8

